@@ -1,129 +1,110 @@
 package Mojolicious::Plugin::Util::RandomString;
 use Mojo::Base 'Mojolicious::Plugin';
+use Mojo::IOLoop;
 use Session::Token;
 
-our $VERSION = '0.08_1';
+our $VERSION = 0.01;
+
+my (%generator, %default, %setting);
+my $read_config;
 
-our (%generator, %setting, %default, %param);
-our $read_config;
-our $ok = 1;
 
 # Register plugin
 sub register {
   my ($plugin, $mojo, $param) = @_;
 
-  $ok--;
+  $param //= {};
 
   if (ref $param ne 'HASH') {
     $mojo->log->fatal(__PACKAGE__ . ' expects a hash reference');
     return;
   };
 
-  if ($param) {
-    $param{$_} = $param->{$_} foreach keys %$param;
-  };
-
-
   # Load parameter from Config file
   unless ($read_config) {
     if (my $config_param = $mojo->config('Util-RandomString')) {
-      %param =  ( %$config_param, %param );
+      $param = { %$config_param, %$param };
     };
     $read_config = 1;
   };
 
-
   # Reseed on fork
   Mojo::IOLoop->timer(
     0 => sub {
       my %created = ();
 
       # Create generators by param
-      foreach (keys %param) {
+      foreach (keys %$param) {
 
-        # Named generator
-        if (ref $param{$_} && ref $param{$_} eq 'HASH') {
+	# Named generator
+	if (ref $param->{$_} && ref $param->{$_} eq 'HASH') {
 
-          next if $created{$_};
+	  next if $created{$_};
 
-          # Construct object
-          unless ($generator{$_} = Session::Token->new(
-            %{ $param{$_} }
-          )) {
+	  # Construct object
+	  unless ($generator{$_} = Session::Token->new(
+	    %{ $param->{$_} }
+	  )) {
 
-            # Unable to construct object
-            $mojo->log->fatal(qq!Unable to create generator for "$_"!);
-            next;
-          };
-          $setting{$_} = { %{$param{$_}} };
-          $created{$_} = 1;
-        }
+	    # Unable to construct object
+	    $mojo->log->fatal(qq!Unable to create generator for "$_"!);
+	    next;
+	  };
+	  $setting{$_} = { %{ $param->{$_} } };
+	  $created{$_} = 1;
+	}
 
-        # Default parameter
-        else {
-          $default{$_} = $param{$_};
-        };
+	# Default parameter
+	else {
+	  $default{$_} = $param->{$_};
+	};
       };
 
-      # Plugin registered
-      $ok++;
-
       # Create default generator
-      unless (exists $generator{default}) {
-        $generator{default} = Session::Token->new( %default );
-      };
+      $generator{'default'} //= Session::Token->new( %default );
     });
 
 
   # Establish 'random_string' helper
   $mojo->helper(
     random_string => sub {
-      my $c = shift;
-      my $gen = $_[0];
 
-      # One tick for loop until the plugin is registered
-      Mojo::IOLoop->one_tick until Mojo::IOLoop->is_running || $ok > 0;
+      # Start Loop unless it is running
+      Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
+      my $gen = $_[1];
 
       # Generate from generator
-      unless ($_[1]) {
+      unless ($_[2]) {
 
-        # Generator doesn't exist
-        if ($gen && !exists $generator{$gen}) {
-          $c->app->log->warn(qq!RandomString generator "$gen" is unknown!);
-          return '';
-        };
+	# Generator doesn't exist
+	if ($gen && !exists $generator{$gen}) {
+	  $mojo->log->warn(qq!RandomString generator "$gen" is unknown!);
+	  return '';
+	};
 
-        # Get from generator
-        return $generator{$gen || 'default'}->get;
+	# Get from generator
+	return $generator{$gen // 'default'}->get;
       };
 
-      $gen = @_ % 2 ? shift : 'default';
+      # Controller
+      shift;
 
-      # Overwrite configuration
-      my %overwrite = @_;
+      # Overwrite default configuration
+      return Session::Token->new(%default, @_)->get unless @_ % 2;
 
-      if (exists $overwrite{entropy}) {
-        $overwrite{length} //= undef;
-      }
-      elsif (exists $overwrite{length}) {
-        $overwrite{entropy} //= undef;
-      };
-
-      # Take default configuration
-      if ($gen eq 'default') {
-        return Session::Token->new(%default, %overwrite)->get;
-      }
+      $gen = shift;
+      return Session::Token->new(%default, @_)->get if $gen eq 'default';
 
       # Overwrite specific configuration
-      elsif ($setting{ $gen }) {
-        return Session::Token->new( %{ $setting{ $gen } }, %overwrite)->get;
+      if ($setting{ $gen }) {
+	return Session::Token->new( %{ $setting{ $gen } } , @_)->get;
       };
 
       # Generator is unknown
-      $c->app->log->warn(qq!RandomString generator "$gen" is unknown!);
+      $mojo->log->warn(qq!RandomString generator "$gen" is unknown!);
       return '';
     }
-  ) unless exists $mojo->renderer->helpers->{random_string};
+  );
 };
 
 
@@ -171,7 +152,7 @@ L<Mojolicious::Plugin::Util::RandomString> is a plugin to generate
 random strings for session tokens, encryption salt, temporary
 password generation etc. Internally it uses L<Session::Token>
 (see L<this comparison|http://neilb.org/reviews/passwords.html#Session::Token>
-for reasons for this decision).
+for reasons of this decision).
 
 This plugin will automatically reseed the random number generator in
 a forking environment like Hypnotoad (although it is untested in other
@@ -198,7 +179,7 @@ L<Mojolicious::Plugin> and implements the following new one.
     }
   };
 
-  # Or in your config file
+  Or in your config file
   {
     'Util-RandomString' => {
       entropy => 256,
@@ -215,7 +196,7 @@ Expects a hash reference containing parameters as defined in
 L<Session::Token> for the default generator.
 To specify named generators, use a name key (other than C<alphabet>,
 C<length>, and C<entropy>) and specify the parameters as a hash reference.
-The name key C<default> can overwrite the default configuration.
+The name key 'default' can overwrite the default configuration.
 
 All parameters can be set either on registration or
 as part of the configuration file with the key C<Util-RandomString>.
@@ -267,9 +248,9 @@ L<Session::Token>.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2013-2021, L<Nils Diewald|http://nils-diewald.de/>.
+Copyright (C) 2013, L<Nils Diewald|http://nils-diewald.de/>.
 
 This program is free software, you can redistribute it
-and/or modify it under the terms of the Artistic License version 2.0.
+and/or modify it under the same terms as Perl.
 
 =cut
